/*
 *
 * Copyright (c) 2007, Sun Microsystems, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of Sun Microsystems nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package example.bluetooth.demo;

import java.io.InputStream;
import java.io.OutputStream;

import javax.bluetooth.LocalDevice;
import javax.bluetooth.UUID;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import javax.microedition.io.StreamConnectionNotifier;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.StringItem;
import javax.microedition.midlet.MIDlet;

/**
 * Contains the Bluetooth API demo, that allows to download the specific images
 * from the other devices.
 * 
 * @version ,
 */
public final class DemoMIDlet extends MIDlet implements CommandListener
{
    /** The messages are shown in this demo this amount of time. */
    static final int ALERT_TIMEOUT = 2000;

    /** A list of menu items */
    private static final String[] elements = { "Phone is Server", "Phone is Client" };

    /** Soft button for exiting the demo. */
    private final Command EXIT_CMD = new Command("Exit", Command.EXIT, 2);

    /** Soft button for launching a client or sever. */
    private final Command OK_CMD = new Command("Ok", Command.SCREEN, 1);

    /** A menu list instance */
    private final List menu = new List("Bluetooth Demo", List.IMPLICIT, elements, null);

    /** value is true after creating the server/client */
    private boolean isInit = false;

    /**
         * Constructs main screen of the MIDlet.
         */
    public DemoMIDlet()
    {
	menu.addCommand(EXIT_CMD);
	menu.addCommand(OK_CMD);
	menu.setCommandListener(this);
    }

    /**
         * Creates the demo view and action buttons.
         */
    public void startApp()
    {
	if (!isInit)
	{
	    show();
	}
    }

    /**
         * Destroys the application.
         */
    protected void destroyApp(boolean unconditional)
    {
    }

    /**
         * Does nothing. Redefinition is required by MIDlet class.
         */
    protected void pauseApp()
    {
    }

    /**
         * Responds to commands issued on "client or server" form.
         * 
         * @param c
         *                command object source of action
         * @param d
         *                screen object containing the item action was performed
         *                on
         */
    public void commandAction(Command c, Displayable d)
    {
	if (c == EXIT_CMD)
	{
	    destroyApp(true);
	    notifyDestroyed();

	    return;
	}

	Form mainScreen = new Form("Javelin client");
	StringItem si = null;

	switch (menu.getSelectedIndex())
	{
	case 0: // Javelin = client, phone = server
	    // imageServer = new GUIImageServer(this);
	    Display.getDisplay(this).setCurrent(mainScreen);
	    si = new StringItem("Serveur mode", null);
	    mainScreen.append(si);
	    try
	    {
		LocalDevice localDevice = LocalDevice.getLocalDevice();

		// prepare a URL to create a notifier
		StringBuffer url = new StringBuffer("btspp://localhost:");

		// add the UUID to identify this service
		url.append(new UUID("1101", false).toString());

		// add the name for our service
		url.append(";authenticate=false;authorize=false;encrypt=false;name=test");

		// create notifier now
		StreamConnectionNotifier notifier = (StreamConnectionNotifier) Connector.open(url.toString());

		localDevice.getRecord(notifier);

		notifier.acceptAndOpen();

		si = new StringItem("C'est pass!", null);
		mainScreen.append(si);
	    }
	    catch (Exception e)
	    {
	    }

	    break;

	case 1: // Javelin = server, phone = client
	    Display.getDisplay(this).setCurrent(mainScreen);
	    si = new StringItem("Client mode", null);
	    mainScreen.append(si);
	    try
	    {
		String url = "btspp://000C840065E1:1";

		StreamConnection con = (StreamConnection) Connector.open(url, Connector.READ_WRITE, true);
		OutputStream output = con.openOutputStream();
		InputStream input = con.openInputStream();
		StringBuffer buff = new StringBuffer(55);

		for (int i = 0; i < 52; i++)
		{
		    output.write(65 + i % 26);
		    Thread.sleep(10);
		}
		output.flush();
		Thread.sleep(100);
		while (input.available() > 0)
		{
		    buff.append((char) input.read());
		    Thread.sleep(20);
		}
		si = new StringItem("Received data: " + buff.toString(), null);
		mainScreen.append(si);
		output.flush();
		output.close();
		input.close();
		con.close();
	    }
	    catch (Exception e)
	    {
	    }

	    // imageClient = new GUIImageClient(this);

	    break;

	default:
	    System.err.println("Unexpected choice...");

	    break;
	}

	isInit = true;
    }

    /** Shows main menu of MIDlet on the screen. */
    void show()
    {
	Display.getDisplay(this).setCurrent(menu);
    }

    /**
         * Returns the displayable object of this screen - it is required for
         * Alert construction for the error cases.
         */
    Displayable getDisplayable()
    {
	return menu;
    }
} // end of class 'DemoMIDlet' definition
